//      WaterSimDCDC Regional Water Demand and Supply Model Version 5.0

//       This is the C# interface to the WaterSim_DCDC FORTRAN dll.

//       Copyright (C) 2011 , The Arizona Board of Regents
//              on behalf of Arizona State University

//       All rights reserved.

//       Developed by the Decision Center for a Desert City
//       Lead Model Development - David A. Sampson <david.a.sampson@asu.edu>

//       This program is free software: you can redistribute it and/or modify
//       it under the terms of the GNU General Public License version 3 as published by
//       the Free Software Foundation.

//       This program is distributed in the hope that it will be useful,
//       but WITHOUT ANY WARRANTY; without even the implied warranty of
//       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//       GNU General Public License for more details.

//       You should have received a copy of the GNU General Public License
//       along with this program.  If not, please see <http://www.gnu.org/licenses/>.
//
//====================================================================================
using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;       // for DllImport
using System.IO;                            // for StreamWriter
using System.Diagnostics;
using System.ComponentModel;
using System.Threading;
using System.Windows.Forms;
using System.Data;
using System.Data.OleDb;
using System.Data.Common;
using System.Web;

namespace WaterSimDCDC
{
    //using WaterSim_SRVDataSetTableAdapters;

    internal class WaterSimU

    {
        protected string mBuild = "base class";
        internal virtual string Build
        { get { return mBuild; } }

        protected string mTestVersion = "Base test";
        internal virtual string TestVersion
        { 
            get 
            {
                string TMonth = TestMonth_().ToString();
                string TDay = TestDay_().ToString();
                string THour = TestHour_().ToString();
                string TMin = TestMin_().ToString();
                mTestVersion = TMonth + "." + TDay + "." + THour + "." + TMin;
                return mTestVersion; 
            } 
        }

        protected int mStatus = 0;
        internal virtual int Status
        { get { return mStatus; } }

        private string _programName;
        private string _arguments;
        private string _dataDirectoryName;
        private string _tempDirectoryName;

        #region constants
            internal const int cNumModeledProviders = 33;
            internal const int cNumFortranProviders = 35;
            internal const int cNumProviders = 38;

            private bool start_ = true;
            //private int endyear = 2085;
            private int startyear = 2006;
            private int minStopYearDrought = 2085;
            private const int OutdoorWaterUsePct = 50;
            internal const int ReclaimedToOutputPct = 10;
            private const int ReclaimedToInputPct = 10;
            private const int ReclaimedOutdoorUsePct = 100;
            private const int ROReclaimedToOutputPct = 80;
            private const int SurfaceWaterToWBankAmt = 1000;
            private const int SurfaceWaterToVadoseAmt = 1000;
            private const int TimeLagForVadoseYears = 12;
            private const int WasteWaterToEffluentPct = 100;
            private const int WaterSupplyToDirectInjectAmt=1000;
            private const int WaterSupplyToResidentialPct = 70;
            private const int WaterSupplyToCIOpct = 30;
        #endregion

        //
        #region Connection to model

        internal WaterSimU(string Data, string Outputs, bool Write)
        {
            // Load the DLL by making a call to it- 
            mBuild = "WaterSim " + BuildNum_().ToString() + ".0" + BuildVer_();
            mBuild = mBuild + "_" + TestVersion;

            //
            Console.WriteLine(mBuild + " =Current Build Release ");
            Console.WriteLine(TestVersion + " =Time Stamp of Release ");
            System.Threading.Thread.Sleep(2000);

            const int cBufferLengthSpecification = 200;
            int sel = 1;
            SetStringData_(ref sel, StringToFixedBytes(cBufferLengthSpecification,
                  Data));

            SetStringOutput_(ref sel, StringToFixedBytes(cBufferLengthSpecification,
                  Outputs));
            // ------------------------------
            WriteToDisk = Write;
            OpenFiles_();
            int mStatus = getStatus_();
            // ------------------------------
            _dataDirectoryName = Data;
            _tempDirectoryName = Outputs;
            ResetAll();
        }
        private static byte[] StringToFixedBytes(int bufferLengthSpecification, string str)
        {
            byte[] bytes = new byte[bufferLengthSpecification];
            for (int i = 0; i < bufferLengthSpecification; ++i) bytes[i] = (byte)32;
            if (0 < str.Length)
            {
                if (bufferLengthSpecification < str.Length)

                    str = str.Substring(0, bufferLengthSpecification);

                ASCIIEncoding.ASCII.GetBytes(str).CopyTo(bytes, 0);
            }
            return bytes;
        }

        //[DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int BuildNum_();

        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int BuildVer_();

        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int TestMonth_();
        [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
        protected static extern int TestDay_();
        [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
        protected static extern int TestHour_();
        [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
        protected static extern int TestMin_();


        [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
        protected static extern void SetStringData_(ref int selector, byte[] newValue);

        [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
        protected static extern void SetStringOutput_(ref int selector, byte[] newValue);


        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getStatus_();

        #endregion

        //-----------------------------------------------------------------------------------//

        #region Simulation Duration- Start and End
         internal int SetSimulationStart
        {
            get
            {
                int gvalue = 0;
                return getStartSimulation_(ref gvalue);
            }
            set
            {
             int start = RangeCheckStartYear(value);
             setStartSimulation_(ref start); 
            } 
        }
        //[DllImport(@"WaterSimDCDC_model.dll")]
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl)]
        protected static extern int getStartSimulation_(ref int value);

        [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
        protected static extern void setStartSimulation_(ref int value);
        //
        internal int SetSimulationEnd
        {
            get
            {
                int gvalue = 0;
                return getEndSimulation_(ref gvalue);
            }
            set 
            {
                int end = RangeCheckEndYear(value);
                setEndSimulation_(ref end);
            } 
        }
        [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
        protected static extern int getEndSimulation_(ref int value);

        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setEndSimulation_(ref int value);
        //
        internal bool WriteToDisk
        {
            get
            {
                bool gvalue = false;
                return getWriteStatus_(ref gvalue);
            }
            set
            {
                setWriteStatus_(ref value);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
        protected static extern bool getWriteStatus_(ref bool value);
        [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
        protected static extern void setWriteStatus_(ref bool value);


            #region Control Run
            internal bool StartSimulation
            {
                get { return start_; }
                set { start_ = value; }
            }
            internal int Startyear
            {
                get { return SetSimulationStart; }
                set { SetSimulationStart = value; }
            }

            internal int Endyear
            {
                get { return SetSimulationEnd; }
                set { SetSimulationEnd = value; }
            }
            #endregion

        internal void OpenFiles()
        {
            OpenFiles_();
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void OpenFiles_();

        internal  void CloseFiles()
        {
            CloseFiles_();
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void CloseFiles_();

        internal  void CloseSpecial()
        {
            CloseFilesS_();
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void CloseFilesS_();

        #endregion

        //-----------------------------------------------------------------------------------//

        #region Run Variables

        // Absent: internal  int CurrentTime
        //
        internal  void ResetYear()
        {
            Init_();
            //InitStates_();
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void InitYear_();

        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void SetStartYear_(ref int value);

        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void InitStates_();
        //
        internal  void ResetAll()
        {
            Init_();
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void Init_();
        //
        internal  void RunOneYear()
        {
            RunOneYear_();
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void RunOneYear_();
        //
        //internal void RunOutputs(RunCallbackHandler runCallback,int FirstGrouping, int SecondGrouping)
        internal void RunOutputs(StreamWriter sw, int year, int FirstGrouping, int SecondGrouping)
        {
            //
        }
        //
        internal string ProgramName
        {
            get { return _programName; }
            set { _programName = value; }
        }

        internal string Arguments
        {
            get { return _arguments; }
            set { _arguments = value; }
        }

        internal string DataDirectoryName
        {
            get { return _dataDirectoryName; }
            set { _dataDirectoryName = value; }
        }

        internal string TempDirectoryName
        {
            get { return _tempDirectoryName; }
            set { _tempDirectoryName = value; }
        }
        //internal bool outputs
        //{
        //}
        //internal enum OutputType { Annual = 1, ProviderLevel };
        //internal virtual int RangeCheckOutputs(OutputType value)
        //{
        //    return RangeCheck((int)OutputType.Annual, (int)value,
        //        (int)OutputType.ProviderLevel,
        //        "Output format");
        //}
        //internal virtual OutputType Outputs
        //{ get { NotImplementedException(); return 0; } set { } }
        #endregion
        #region RunCall
        // ====================================================================
        internal  void Run(RunCallbackHandler runCallback)
        {
            // Give the calling program a chance to capture annual values
            if (null != runCallback) runCallback(SetSimulationStart);
            //
            //const int provider = 10;
            //const float maf=1000000;
            string versionFlag = "";
            versionFlag = mBuild;

            DateTime now = DateTime.Now;
            string filename = "test_" + now.Month.ToString() + now.Day.ToString() + now.Year.ToString()
                  + "_" + versionFlag + ".csv";
            StreamWriter sw = new StreamWriter(filename);
            for (int year = Startyear; year < Endyear; ++year)
            {
                RunOneYear();
                GroundWater(year);
                ConsoleWrite(year);
                if (null != runCallback) runCallback(year);
                StartSimulation = false;

            //    sw.Write(year + ", ");
            //    sw.Write(ProviderGWPumpedMunicipal[18] + ",");
            //    sw.Write(ProviderGWdesignations[18] + ",");
            //    sw.Write(VadoseToAquiferFlux[18] + ",");
            //    sw.WriteLine(ProviderGroundwater[18]);
            //     for (int i = 0; i < 32; ++i)
            //    {
            //        //sw.Write(get_ProviderGPCD[i] + ", ");
            //    }
            //    //sw.WriteLine(",");

            }
            //sw.Flush();
            CloseFiles_();
        }
        //
 
        // ====================================================================
        #endregion
        //-----------------------------------------------------------------------------------//
        #region Internally Defined I/O
        internal struct outputs
        {

            internal float TemperatureMultiplier
            {
                get
                {
                    int gvalue = 0;
                    float ClimateDemand = 0;
                    getDemandMultiplier_(ref gvalue);
                    return ClimateDemand = Convert.ToSingle(gvalue * 0.01);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            static extern int getDemandMultiplier_(ref int value);

        }
        #endregion
        //-----------------------------------------------------------------------------------//
        private void ConsoleWrite(int year)
        {
            //Console.WriteLine(year + "  ");

        }
         //
        #region Groundwater
        //private void GroundWater(int year)
        internal void GroundWater(int year)
        {   
            //
            double[] availableGroundwater33 = new double[cNumModeledProviders];
            if (StartSimulation)
            {
                for (int i = 0; i < cNumModeledProviders; ++i)
                {
                    availableGroundwater33[i] = Convert.ToDouble(ProviderGroundwater[i]);
                }
            }
          
            //WaterSimU.TrimProvNamesD38(MF2k.initMODFLOWGroundwater(cNumProviders, DataDirectoryName));

            double saltVerdeAnnualOverflow = 0;
            double sharedRecharge = 0;
            double[] providerGWrecharge = new double[cNumModeledProviders];
            int[] providerGWpumped = new int[cNumModeledProviders];
            int SRPpumping = 0;
            int providerPumped = 0;
            providerGWpumped = ProviderGWPumpedMunicipal;
            for (int i = 0; i < cNumModeledProviders; ++i)
            {
                SRPpumping += SaltVerdeClassBCDesignations[i]; // 09.09.10 das
                providerGWrecharge[i] = Convert.ToDouble(ProviderGWRecharge[i]);
                providerPumped += providerGWpumped[i];
            }
            //Console.WriteLine(providerPumped + " Provider pumped");
            // ------------------------------------------------------------------------------
            UpdateAvailableGroundwaterByLayer(
               year, saltVerdeAnnualOverflow, sharedRecharge,
                ref providerGWrecharge, SRPpumping,
                ref providerGWpumped, ref availableGroundwater33);
        }
        private void UpdateAvailableGroundwaterByLayer(
            int year,
            double saltVerdeAnnualOverflow, double sharedRecharge, ref double[] providerRecharges,
            int saltVerdeAnnualGroundwaterDemand, ref int[] groundwaterDeliveries33,
            ref double[] availableGroundwater33)
        {
            //
            double[] GWratio = new double[cNumModeledProviders];
            bool SStart = StartSimulation;
            //
            int[] providerGroundwaterDeliveriesTotalSet = new int[cNumProviders];

            double[] dProviderRechargeTotalSet = new double[cNumProviders];

            dProviderRechargeTotalSet = WaterSimU.ExpandProvNames(providerRecharges);
            providerGroundwaterDeliveriesTotalSet = WaterSimU.ExpandProvNamesInt(groundwaterDeliveries33);
            //
            double[] availableGroundwater38 = new double[cNumProviders];
            availableGroundwater38 = WaterSimU.ExpandProvNames(availableGroundwater33);
            //
            for (int iProv = 0; iProv < cNumProviders; ++iProv)
            {
                availableGroundwater38[iProv] +=
                   dProviderRechargeTotalSet[iProv] - providerGroundwaterDeliveriesTotalSet[iProv];

            }
            for (int iProv = 0; iProv < cNumModeledProviders; ++iProv)
            {
                //Console.WriteLine(year + "  " + iProv + "  " + groundwaterDeliveries33[iProv]
                //     + "  " + GWratio[iProv]);

            }
            //Console.WriteLine("In Abbreviated Groundwater: for " + year);

            //Console.WriteLine(year + "  " + iProv + "  " + popGrowthPct + "  "
            //    + iSalt + "  "
            //    + iColo + "  "
            //    + SRV.ProviderAbbreviations[iProv] + "  "
            //    + _model.Populations[iProv] + "  ");


            // Return the 38 to the 33 water providers of focus
            double[] Groundwater = WaterSimU.TrimProvNamesD38(availableGroundwater38);
            for (int iProv = 0; iProv < cNumModeledProviders; ++iProv)
            {
                availableGroundwater33[iProv] = Groundwater[iProv];
            }

        }
        #endregion
        //-----------------------------------------------------------------------------------//
        #region Colorado River Variables [all certified]
        //
        internal  int ColoradoHistoricalData
        {
            get
            {
                int gvalue = 0;
                return getCOHistData_(ref gvalue);
            }
            set
            {
                int year = RangeCheckColoradoData(value);
                setCOHistData_(ref year);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
        protected static extern int getCOHistData_(ref int value);

        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setCOHistData_(ref int value);




        // Index year for runoff estimates (from historical runoff data)

        //          [CERTIFIED 02.02.2010 das]
        internal  int ColoradoHistoricalExtractionStartYear
        {

            get
            {
                int gvalue = 0;
                return getCOHistExtractionStartYear_(ref gvalue);
            }
            set
            {
                int year = RangeCheckColoradoHistoricalExtractionStartYear(value);
                setCOHistExtractionStartYear_(ref year);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getCOHistExtractionStartYear_(ref int value);

        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setCOHistExtractionStartYear_(ref int value);
        //

        // Drought year start
        //
        //          [CERTIFIED 02.02.2010 das]
        internal  int ColoradoUserAdjustmentStartYear
        {
            get
            {
                return getCOUserAdjustmentStartYear_();
            }
            set
            {
                int year = RangeCheckColoradoUserAdjustmentStartYear(value);
                setCOUserAdjustmentStartYear_(ref year);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getCOUserAdjustmentStartYear_();
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setCOUserAdjustmentStartYear_(ref int value);
        //


        // Drought year end
        //
        //          [CERTIFIED 02.02.2010 das]
        internal  int ColoradoUserAdjustmentStopYear
        {
            get
            {
                return getCOUserAdjustmentStopYear_();
            }
            set
            {
                int year = RangeCheckColoradoUserAdjustmentStopYear(value);
                setCOUserAdjustmentStopYear_(ref year);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getCOUserAdjustmentStopYear_();
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setCOUserAdjustmentStopYear_(ref int value);
        //

        // Drought factor (%) to adjust CO River runoff estimates (i.e., increase or reduce)
        //
        //              [CERTIFIED 02.02.2010 das]
        internal  int ColoradoUserAdjustmentPercent
        {
            get
            {
                return getCOUserAdjustmentPercent_();
            }
            set
            {
                int percent = RangeCheckColoradoUserAdjustmentPercent(value);
                setCOUserAdjustmentPercent_(ref percent);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getCOUserAdjustmentPercent_();
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setCOUserAdjustmentPercent_(ref int value);
        //

        //
        // Climate factor (%) to adjust CO River runoff estimates (i.e., increase or reduce)
        //
        //               [CERTIFIED 02.02.2010 das]
        internal  int ColoradoClimateAdjustmentPercent
        {
            get
            {
                int gvalue = 0;
                return getCOClimateAdjustmentPercent_(ref gvalue);
            }
            set
            {
                int percent = RangeCheckColoradoClimateAdjustmentPercent(value);
                setCOClimateAdjustmentPercent_(ref percent);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getCOClimateAdjustmentPercent_(ref int value);
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setCOClimateAdjustmentPercent_(ref int value);
        //
        // absent: internal  int ColoradoActualEstimateOrRandom (NOT used)
        //
        #endregion CO river variables
        //-----------------------------------------------------------------------------------//
        #region Salt/Verde River Variables [all certified]
        //
        internal  int SaltVerdeTontoHistoricalData
        {
            get
            {
                int gvalue = 0;
                return getSVTHistData_(ref gvalue);
            }
            set
            {
                int gvalue = RangeCheckSVTData(value);
                setSVTHistData_(ref gvalue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getSVTHistData_(ref int value);

        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setSVTHistData_(ref int value);

        //   Index year for the Salt-Verde-Tonto rivers.  The year to start simulations 
        // from historical data estimates.
        //
        //              [CERTIFIED 02.02.2010 das]
        internal  int SaltVerdeHistoricalExtractionStartYear
        {
            get
            {
                int gvalue = 0;
                return getSVHistExtractionStartYear_(ref gvalue);
            }
            set
            {
                int year = RangeCheckSaltVerdeHistoricalExtractionStartYear(value);
                setSVHistExtractionStartYear_(ref year);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getSVHistExtractionStartYear_(ref int value);

        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setSVHistExtractionStartYear_(ref int value);
        //

        // Drought year - year to implement a drought on the Salt-Verde-Tonto River system
        //
        //           [CERTIFIED 02.02.2010 das]
        internal  int SaltVerdeUserAdjustmentStartYear
        {
            get
            {
                return getSVUserAdjustmentStartYear_();
            }
            set
            {
                int year = RangeCheckSaltVerdeUserAdjustmentStartYear(value);
                setSVUserAdjustmentStartYear_(ref year);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getSVUserAdjustmentStartYear_();
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setSVUserAdjustmentStartYear_(ref int value);
        //

        // Year to stop the imposed drought on the Salt-Verde-Tonto river system
        //
        //           [CERTIFIED 02.02.2010 das]
        internal  int SaltVerdeUserAdjustmentStopYear
        {
            get
            {
                return getSVUserAdjustmentStopYear_();
            }
            set
            {
                int year = RangeCheckSaltVerdeUserAdjustmentStopYear(value);
                setSVUserAdjustmentStopYear_(ref year);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getSVUserAdjustmentStopYear_();
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setSVUserAdjustmentStopYear_(ref int value);
        //

        // Reductions in base flow of the Salt-Verde-Tonto based on the drought imposed above
        //
        //                    [CERTIFIED 02.02.2010 das]
        internal  int SaltVerdeUserAdjustmentPercent
        {
            get
            {
                return getSVUserAdjustmentPercent_();
            }
            set
            {
                int percent = RangeCheckSaltVerdeUserAdjustmentPercent(value);
                setSVUserAdjustmentPercent_(ref percent);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getSVUserAdjustmentPercent_();
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setSVUserAdjustmentPercent_(ref int value);
        //

        //   Climate factor for the Salt-Verde-Tonto river system (proportional increase in flow
        // as influenced by the adjustment imposed).  Used as a percent in the Interface.
        //
        //                   [CERTIFIED 02.02.2010 das]
        internal  int SaltVerdeClimateAdjustmentPercent
        {
            get
            {
                int gvalue = 0;
                return getSVClimateAdjustmentPercent_(ref gvalue);
            }
            set
            {
                int percent = RangeCheckSaltVerdeClimateAdjustmentPercent(value);
                setSVClimateAdjustmentPercent_(ref percent);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getSVClimateAdjustmentPercent_(ref int value);
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setSVClimateAdjustmentPercent_(ref int value);
        //
        // absent:  internal  SRPReleaseEstimateMethodType SRPReleaseEstimateMethod
        //
        //   Three approaches to estimate SRP release from the Salt-Verde-Tonto River reservoir. The
        // historical estimates are not verified (original code).  The Tim Lant approach, and the
        // David Sampson Approach.  See draft manusctip for details.  Defult is now the DAS approach.

        #endregion
        //-----------------------------------------------------------------------------------//
        #region Groundwater Variables: County-scale (inputs and outputs) [all certified]
        // ===================================
        // I need to be reminded why I did this.....

        //

        // Current state of the groundwater-county level simulations (acft)
        //
        //           [CERTIFIED 02.03.2010 das]
        internal  int GroundwaterState
        {
            get
            {
                int gvalue = 0;
                return getGroundwaterState_(ref gvalue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getGroundwaterState_(ref int value);
        //

        // absent:   internal  int NaturalRecharge
        //
        //      [CERTIFIED 02.03.2010 das]
        internal  int NaturalRecharge
        {
            get
            {
                int gvalue = 0;
                return getNaturalRecharge_(ref gvalue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getNaturalRecharge_(ref int value);

        //

        //  Annual difference in groundwater state variable - county scale: acft a-1
        //
        //          [CERTIFIED 02.03.2010 das]
        internal  int GroundwaterDifference
        {
            get
            {
                int gvalue = 0;
                return getGroundwaterDiff_(ref gvalue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getGroundwaterDiff_(ref int value);
        //

        // Annual groundwater pumped, county scale (acft a-1)
        //
        //          [CERTIFIED 02.03.2010 das]
        internal  int GroundwaterPumpage
        {
            get
            {
                int gvalue = 0;
                return getGroundwaterPumpage_(ref gvalue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getGroundwaterPumpage_(ref int value);
        //

        // Annual groundwater recharged, county scale (acft a-1)
        //
        //          [CERTIFIED 02.03.2010 das]
        internal  int GroundwaterRecharge
        {
            get
            {
                int gvalue = 0;
                return getGroundwaterRecharge_(ref gvalue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getGroundwaterRecharge_(ref int value);
        //
        #endregion
        //-----------------------------------------------------------------------------------//
        #region Energy

        //          [CERTIFIED 02.11.2010 das] - Giga Watt hours a-1
        internal  int CAPEnergyUsedInPumping
        {
            get
            {
                int gvalue = 0;
                return getCAPEnergyUsedInPumping_(ref gvalue);
                //return gvalue;
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getCAPEnergyUsedInPumping_(ref int value);

        //

        //
        // absent: internal  int GroundwaterPumpingEnergyUse

         #endregion
        //-----------------------------------------------------------------------------------//
        #region Population variables
        //
        //  Maricopa County population from MAG projections 
        //
        //      [CERTIFIED- 01.29.10 das]
        internal  int Population
        {
            get
            {
                int gvalue = 0;
                return getPopMaricopa_(ref gvalue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getPopMaricopa_(ref int value);
        //
        //   Factor to increase or decrease the county population.  This variable uses the
        // annual differences in base projection to add to the previous year estimate to obtain
        // a new estimate (i.e., pop in year -1 + (pop in year - pop in year-1)* PopulationGrowthRateAdjustment).
        //
        //          [CERTIFIED- 01.29.10 das]
        internal  int PopulationGrowthRateAdjustmentPercent
        {
            get
            {
                int gvalue = 0;
                return getPopGrowthRateAdjustmentPct_(ref gvalue);
            }
            set
            {
                int percent = RangeCheckPopulationGrowthRateAdjustmentPercent(value);
                setPopGrowthRateAdjustmentPct_(ref percent);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setPopGrowthRateAdjustmentPct_(ref int value);

        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getPopGrowthRateAdjustmentPct_(ref int value);
        //
        // absent:  internal  int PopulationIn2030
        //
        // -------------------------------------------------------
        // Provider-level variables
        //
        // Provider-specific population growth rate adjustments.
        //
        internal  int[] get_ProviderPopGrowthRateAdjustPct
        {
            get
            {
                int numProviders = cNumFortranProviders;
                int[] PopulationGrowthRateAdjustPct = new int[numProviders];
                getPopGrowthRateAdjustPct_(ref numProviders, PopulationGrowthRateAdjustPct);
                return TrimProvNames(PopulationGrowthRateAdjustPct);
            }
            set
            {
                int num = cNumFortranProviders;
                citiModel = value;
                setPopGrowthRateAdjustPct_(ref num, ExpandProvNames35(citiModel));
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getPopGrowthRateAdjustPct_(ref int count, int[] values);

        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setPopGrowthRateAdjustPct_(ref int count, int[] values);
        //
        //  Water Provider Populations     
        //
        //      [CERTIFIED- 01.29.10 das][RE-CERTIFIED- 02.04.10 das]
        internal  int[] Populations
        {
            get
            {
                int num = cNumFortranProviders;
                int[] Populations = new int[num];
                getPopulations_(ref num, Populations);
                return TrimProvNames(Populations);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getPopulations_(ref int count, int[] values);

        #endregion
        //-----------------------------------------------------------------------------------//
        #region Policy Variables [all certified-of those of interest]
        //
        internal int ReduceGPCDpct
        {
            set
            {
                int checkedValue = RangeCheckGPCDpct(value);
                setReductionGPCDPct_(ref checkedValue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setReductionGPCDPct_(ref int value);


        // absent:  internal  RunoffDataTypeCO RunoffDataCO

        // absent:  internal  RunoffDataTypeSVT RunoffDataSVT

        // ------------------------------------------------------------
        // County Scale

        //
        // County-scale water policies
        //
        internal enum WaterPolicyType { NoOverdraft = 1, SatisfyDemand, FiveYearSustainable, FixedResidentialGPCD };
        //           [CERTIFIED- 02.03.10 das]
        internal WaterPolicyType WaterPolicy
        {
            get
            {
                int iWaterPolicy = getWaterPolicy_();
                switch (iWaterPolicy)
                {
                    case 1:
                        return WaterPolicyType.NoOverdraft;
                    case 2:
                        return WaterPolicyType.SatisfyDemand;
                    case 3:
                        return WaterPolicyType.FiveYearSustainable;
                    case 4:
                        return WaterPolicyType.FixedResidentialGPCD;
                }
                throw new Exception("Unknown water policy type: " + iWaterPolicy.ToString());
            }
            set
            {
                int checkedValue = RangeCheckWaterPolicy(value);
                setWaterPolicy_(ref checkedValue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
        protected static extern int getWaterPolicy_();

        [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
        protected static extern void setWaterPolicy_(ref int value);


        // Year to implement policy
        //
        //        [CERTIFIED- 02.03.10 das]
        internal  int WaterPolicyStartYear
        {
            get
            {
                return getWaterPolicyStartYear_();
            }
            set
            {
                int year = RangeCheckWaterPolicyStartYear(value);
                setWaterPolicyStartYear_(ref year);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getWaterPolicyStartYear_();

        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setWaterPolicyStartYear_(ref int value);
        //

        //  Shortage sharing of water as imposed by policy
        //
        //                [CERTIFIED- 02.03.10 das]
        internal ShortageSharingPolicyType ShortageSharingPolicy
        {
            get
            {
                int gvalue = 0;
                int iShortageSharingPolicy = getShortagePolicy_(ref gvalue);
                return 1 == iShortageSharingPolicy ?
                    ShortageSharingPolicyType.ProportionalSharing :
                    ShortageSharingPolicyType.AgLosesWaterFirst;
            }
            set
            {
                int checkedValue = RangeCheckShortageSharingPolicy(value);
                setShortagePolicy_(ref checkedValue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getShortagePolicy_(ref int value);

        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setShortagePolicy_(ref int value);
        //

        // Residential GPCD 
        //
        //        [CERTIFIED- 02.03.10 das]
        internal  int ResidentialGPCD
        {
            get
            {
                //int something = getResidentialGPCD_();
                //Console.WriteLine(something.ToString());
                //return something;
                return getResidentialGPCD_();
            }
            set
            {
                int checkedValue = RangeCheckResidentialGPCD(value);
                setResidentialGPCD_(ref checkedValue);
                checkedValue = Convert.ToInt32(checkedValue * 1.33);
                setTotalGPCD_(ref checkedValue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getResidentialGPCD_();
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setResidentialGPCD_(ref int value);
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setTotalGPCD_(ref int value);
        //

        // Three residential density paths (1-3)
        //
        //            [CERTIFIED- 02.03.10 das]
        internal  int ResidDensityPath
        {
            get
            {
                int gvalue = 0;
                return getResidDensityPath_(ref gvalue);
            }
            set
            {
                int checkedValue = RangeCheckResidDensityPath(value);
                setResidDensityPath_(ref checkedValue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getResidDensityPath_(ref int value);
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setResidDensityPath_(ref int value);
        //

        //   Household percent pools.  In the model used to calculate the difference
        // between current and the year 2000 estimate (default for 2000 is 20%)
        //
        //            [CERTIFIED- 02.03.10 das]
        internal  int HHoldPctPools
        {
            set
            {
                int checkedValue = RangeCheckHHoldPct(value);
                setPropPools_(ref checkedValue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setPropPools_(ref int value);
        //

        //   Household percent mesic.  In the model used to calculate the difference
        // between current and the year 2000 estimate (default for 2000 is 20%)
        //
        //        [CERTIFIED- 02.03.10 das]
        internal  int HHoldPctMesic
        {
            set
            {
                int checkedValue = RangeCheckHHoldPct(value);
                setPropMesic_(ref checkedValue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setPropMesic_(ref int value);
        //

        //   Agricultural retirement year, when all but Indian agriculture stops. Default
        // is 2070.
        //
        //               [CERTIFIED- 02.03.10 das]
        internal  int AgricultureRetirementYear
        {
            get
            {
                int gvalue = 0;
                return getAgricultureRetirementYear_(ref gvalue);
            }
            set
            {
                int year = RangeCheckAgricultureRetirementYear(value);
                setAgricultureRetirementYear_(ref value);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getAgricultureRetirementYear_(ref int value);
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setAgricultureRetirementYear_(ref int value);
        //
        // absent: internal  int GWCutoffSwitch
        //

        //
        // absent:  internal  int MirrorWaterPolicy
        //
        #region Ground Water Policy Variables
        //
        // absent: internal  int ArizonaWaterBankingAuthoritySwitch 
        //
        // absent:  internal  int CAGRDSwitch
        //
        //   [CERTIFIED- 02.09.10 das]
        internal  bool NaturalRechargeSwitchF
        {
            get
            {
                bool gvalue = false;
                return getNaturalRechargeSwitch_(ref gvalue);
            }
            set
            {
                setNaturalRechargeSwitch_(ref value);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern bool getNaturalRechargeSwitch_(ref bool value);
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setNaturalRechargeSwitch_(ref bool value);

        //
        // absent:   internal  int GroundWaterCutoffThreshold
        //
        #endregion
        #region Recycled Water Policy Variables
        //
        // absent:  internal  int RecycledWaterRatePercent 
        //
        // absent:  internal  int ResidentialIndoorUsePercent
        //
        // absent:   internal  int CommercialAndIndustrialIndoorUsePercent
        //
        //
         #endregion

        #endregion
        //-----------------------------------------------------------------------------------//
        #region Policy-Provider Inputs

        internal int[] TimeLagVadoseYears
        {
            get
            {
                int num = cNumFortranProviders;
                int[] TimeLagVadoseYears = new int[num];
                getTimeLagVadoseYears_(ref num, TimeLagVadoseYears);
                return WaterSimU.TrimProvNameS(TimeLagVadoseYears);
            }
            set
            {
                int pdefault = TimeLagForVadoseYears;
                int num = cNumFortranProviders;
                citiModel = value;
                setTimeLagVadoseYears_(ref pdefault, ref num, ExpandProvNames35(citiModel));
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
        protected static extern int getTimeLagVadoseYears_(ref int count, int[] values);

        [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
        protected static extern void setTimeLagVadoseYears_(ref int pdefault ,ref int count, int[] values);

        // 1 = read from file (on disk)
        // 2 = six-year average GPCD used to adjust new provider demand based on 2006 GPCD by provider (silly at this point)
        // 3 = SES estimate of GPCD with population
        // 4 = SES and interface changes to GPCD
        internal  int ProviderDemandOption
        {
            get
            {
                int gvalue = 0;
                return getDemandOption_(ref gvalue);
            }
            set
            {
                int checkedValue = RangeCheckDemandOption(value);
                setDemandOption_(ref checkedValue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
        protected static extern int getDemandOption_(ref int value);

        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setDemandOption_(ref int value);
        //
        internal int DemandPercent
        {
            get
            {
                int gvalue = 0;
                return getDemandPercent_(ref gvalue);
            }
            set
            {
                int gvalue = RangeCheckDemandPercent(value);
                setDemandPercent_(ref gvalue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getDemandPercent_(ref int value);
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setDemandPercent_(ref int value);
        //
        internal int[] get_WaterBankingOption
        {
            get
            {
                int num = cNumFortranProviders;
                int[] get_WaterBankingOption = new int[num];
                getWaterBankingOption_(ref num, get_WaterBankingOption);
                return WaterSimU.TrimProvNameS(get_WaterBankingOption);
            }
            set
            {
                int num = cNumFortranProviders;
                citiModel = value;
                setWaterBankingOption_(ref num, ExpandProvNames35(citiModel));
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
        protected static extern void getWaterBankingOption_(ref int count, int[] values);

        [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
        protected static extern void setWaterBankingOption_(ref int count, int[] values);


        #endregion
        //-----------------------------------------------------------------------------------//
        #region testcode
        internal int[] test = new int[33];
        internal int[] Test
        {
            get
            {
                return test;
            }
            set
            {
                test = value;
            }
        }
        #endregion
        //-----------------------------------------------------------------------------------//
        #region CityModel
        #region Parameters
        private int[] citiModel = new int[33];
             internal int[] get_parmWWtoRWWTPpct
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] get_parmWWtoRWWTPpct = new int[num];
                    getParmWWtoRWWTPPct_(ref num, get_parmWWtoRWWTPpct);
                    return WaterSimU.TrimProvNameS(get_parmWWtoRWWTPpct);
                }
                set
                {
                    int num = cNumFortranProviders;
                    citiModel = value;
                    setParmWWtoRWWTPPct_(ref num, ExpandProvNames35(citiModel));
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getParmWWtoRWWTPPct_(ref int count, int[] values);

            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern void setParmWWtoRWWTPPct_(ref int count, int[] values);
        
            //
            internal int[] get_parmReclaimedWWtoROpct
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] get_parmReclaimedWWtoROpct = new int[num];
                    getParmRWWToROPct_(ref num, get_parmReclaimedWWtoROpct);
                    return TrimProvNameS(get_parmReclaimedWWtoROpct);
                }
                set
                {
                    int num = cNumFortranProviders;
                    citiModel = value;
                    setParmRWWToROPct_(ref num,ExpandProvNames35(citiModel));
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getParmRWWToROPct_(ref int count, int[] values);

            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern void setParmRWWToROPct_(ref int count, int[] values);
            ////
            internal int[] get_parmWWtoEffluentPct
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] get_parmWWtoEffluentPct = new int[num];
                    getParmWWtoEffluentPct_(ref num, get_parmWWtoEffluentPct);
                    return TrimProvNameS(get_parmWWtoEffluentPct);

                }
                set
                {
                    int pdefault = WasteWaterToEffluentPct;
                    int num = cNumFortranProviders;
                    citiModel = value;
                    setParmWWtoEffluentPct_(ref pdefault,ref num, ExpandProvNames35(citiModel));
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getParmWWtoEffluentPct_(ref int count, int[] values);

            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern void setParmWWtoEffluentPct_(ref int mydefault,ref int count, int[] values);
            //
            internal int[] get_parmEffluentToVadosePct
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] get_parmEffluentToVadosePct = new int[num];
                    getParmEffluenttoVadosePct_(ref num, get_parmEffluentToVadosePct);
                    return TrimProvNameS(get_parmEffluentToVadosePct);
                }
                set
                {
                    int num = cNumFortranProviders;
                    citiModel = value;
                    setParmEffluenttoVadosePct_(ref num, ExpandProvNames35(citiModel));
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getParmEffluenttoVadosePct_(ref int count, int[] values);

            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern void setParmEffluenttoVadosePct_(ref int count, int[] values);
            //
            internal int[] get_parmEffluentToPowerPlantPct
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] get_parmEffluentToPowerPlantPct = new int[num];
                    getParmEffluentToPPlantPct_(ref num, get_parmEffluentToPowerPlantPct);
                    return TrimProvNames(get_parmEffluentToPowerPlantPct);

                }
                set
                {
                    int num = cNumFortranProviders;
                    citiModel = value;
                    setParmEffluentToPPlantPct_(ref num, ExpandProvNames35(citiModel));
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getParmEffluentToPPlantPct_(ref int count, int[] values);

            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern void setParmEffluentToPPlantPct_(ref int count, int[] values);

            internal int[] get_parmReclaimedToVadosePct
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] get_parmReclaimedToVadosePct = new int[num];
                    getParmReclaimedToVadosePct_(ref num, get_parmReclaimedToVadosePct);
                    return TrimProvNameS(get_parmReclaimedToVadosePct);
                }
                set
                {
                    int numProviders = cNumFortranProviders;
                    citiModel = value;
                    setParmReclaimedToVadosePct_(ref numProviders, ExpandProvNames35(citiModel));
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getParmReclaimedToVadosePct_(ref int count, int[] values);

            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern void setParmReclaimedToVadosePct_(ref int count, int[] values);
            //
            internal int[] get_parmReclaimedToDirectInjectPct
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] get_parmReclaimedToDirectInjectPct = new int[num];
                    getParmReclaimedToDirectInjPct_(ref num, get_parmReclaimedToDirectInjectPct);
                    return TrimProvNameS(get_parmReclaimedToDirectInjectPct);
                }
                set
                {
                    int num = cNumFortranProviders;
                    citiModel = value;
                    setParmReclaimedToDirectInjPct_(ref num,ExpandProvNames35(citiModel) );
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getParmReclaimedToDirectInjPct_(ref int count, int[] values);

            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern void setParmReclaimedToDirectInjPct_(ref int count, int[] values);
            //
            internal bool parmOverrideShortage
            {
                get
                {
                    bool gvalue = false;
                    return getOverrideShortage_(ref gvalue);
                }
                set
                {
                    bool Override = value;
                    setOverrideShortage_(ref Override);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
            protected static extern bool getOverrideShortage_(ref bool value);

            [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
            protected static extern void setOverrideShortage_(ref bool value);


            //
            internal int[] get_parmReclaimedToOutputPct
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] get_parmReclaimedToOutputPct = new int[num];
                    getParmReclaimedToOutputPct_(ref num, get_parmReclaimedToOutputPct);
                    return TrimProvNameS(get_parmReclaimedToOutputPct);
                }
                set
                {
                    int num = cNumFortranProviders;
                    int pdefault = ReclaimedToOutputPct;
                    citiModel = value;
                    setParmReclaimedToOutputPct_(ref pdefault,ref num, ExpandProvNames35(citiModel));
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getParmReclaimedToOutputPct_(ref int count, int[] values);

            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern void setParmReclaimedToOutputPct_(ref int mydefault, ref int count, int[] values);
            //
            internal int[] get_parmReclaimedToInputPct
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] get_parmReclaimedToInputPct = new int[num];
                    getParmReclaimedToInputPct_(ref num, get_parmReclaimedToInputPct);
                    return TrimProvNameS(get_parmReclaimedToInputPct);
                }
                set
                {
                    int num = cNumFortranProviders;
                    int pdefault = ReclaimedToInputPct;
                    citiModel = value;
                    setParmReclaimedToInputPct_(ref pdefault, ref num, ExpandProvNames35(citiModel));
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
            protected static extern int getParmReclaimedToInputPct_(ref int count, int[] values);

            [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
            protected static extern void setParmReclaimedToInputPct_(ref int mydefault, ref int count, int[] values);
            //

            //
            internal int[] get_parmROReclaimedToOutputPct
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] get_parmROReclaimedToOutputPct = new int[num];
                    getParmROToOutputPct_(ref num, get_parmROReclaimedToOutputPct);
                    return TrimProvNameS(get_parmROReclaimedToOutputPct);
                }
                set
                {
                    int num = cNumFortranProviders;
                    int pdefault = ROReclaimedToOutputPct;
                    citiModel = value;
                    setParmROToOutputPct_(ref pdefault, ref num, ExpandProvNames35(citiModel));
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getParmROToOutputPct_(ref int count, int[] values);

            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern void setParmROToOutputPct_(ref int mydefault, ref int count, int[] values);
            //
            //
            internal int[] get_parmSurfaceWaterToWbankPct
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] get_parmSurfaceWaterToWbankPct = new int[num];
                    getParmSWtoWBankPct_(ref num, get_parmSurfaceWaterToWbankPct);
                    return TrimProvNameS(get_parmSurfaceWaterToWbankPct);
                }
                set
                {
                    int num = cNumFortranProviders;
                    citiModel = value;
                    setParmSWtoWBankPct_(ref num, ExpandProvNames35(citiModel));
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getParmSWtoWBankPct_(ref int count, int[] values);

            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern void setParmSWtoWBankPct_(ref int count, int[] values);
            //
            internal int[] get_parmSurfaceWaterToWBankAmt
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] get_parmSurfaceWaterToWBankAmt = new int[num];
                    getParmSWtoWBankAmt_(ref num, get_parmSurfaceWaterToWBankAmt);
                    return TrimProvNameS(get_parmSurfaceWaterToWBankAmt);
                }
                set
                {
                    int num = cNumFortranProviders;
                    int pdefault = SurfaceWaterToWBankAmt; // Acre-feet to begin with default banking
                    citiModel = value;
                    setParmSWtoWBankAmt_(ref pdefault, ref num, ExpandProvNames35(citiModel));
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
            protected static extern int getParmSWtoWBankAmt_(ref int count, int[] values);
 
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern void setParmSWtoWBankAmt_(ref int mydefault,ref int count, int[] values);
            //
            internal int[] get_parmSurfaceWaterToVadoseAmt
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] get_parmSurfaceWaterToVadoseAmt = new int[num];
                    getParmSWtoVadoseAmt_(ref num, get_parmSurfaceWaterToVadoseAmt);
                    return TrimProvNameS(get_parmSurfaceWaterToVadoseAmt);
                }

                set
                {
                    int num = cNumFortranProviders;
                    int pdefault = SurfaceWaterToVadoseAmt; // Acre-feet to befin with default banking
                    citiModel = value;
                    setParmSWtoVadoseAmt_(ref pdefault, ref num, ExpandProvNames35(citiModel));
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
            protected static extern int getParmSWtoVadoseAmt_(ref int count, int[] values);

            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern void setParmSWtoVadoseAmt_(ref int mydefault, ref int count, int[] values);
            //
            internal int[] get_parmWaterSupplyToDirectInjectAmt
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] get_parmWaterSupplyToDirectInjectAmt = new int[num];
                    getParmWStoDIAmt_(ref num, get_parmWaterSupplyToDirectInjectAmt);
                    return TrimProvNameS(get_parmWaterSupplyToDirectInjectAmt);
                }
                set
                {
                    int num = cNumFortranProviders;
                    int pdefault = WaterSupplyToDirectInjectAmt; // Acre-feet to befin with default banking
                    citiModel = value;
                    setParmWStoDIAmt_(ref pdefault, ref num, ExpandProvNames35(citiModel));
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
            protected static extern int getParmWStoDIAmt_(ref int count, int[] values);

            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern void setParmWStoDIAmt_(ref int mydefault, ref int count, int[] values);
            //
            internal int[] get_parmOutdoorWaterUsePct
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] get_parmOutdoorWaterUsePct = new int[num];
                    getParmOutDoorPct_(ref num, get_parmOutdoorWaterUsePct);
                    return TrimProvNameS(get_parmOutdoorWaterUsePct);
                }
                set
                {
                    int num = cNumFortranProviders;
                    int pdefault = OutdoorWaterUsePct;
                    citiModel = value;
                    setParmOutDoorPct_(ref pdefault,ref num, ExpandProvNames35(citiModel));
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getParmOutDoorPct_(ref int count, int[] values);

            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern void setParmOutDoorPct_(ref int mydefault,ref int count, int[] values);
            //
            internal int[] get_parmReclaimedOutdoorUsePct
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] get_parmReclaimedOutdoorUsePct = new int[num];
                    getReclaimedOutdoorUse_(ref num, get_parmReclaimedOutdoorUsePct);
                    return TrimProvNameS(get_parmReclaimedOutdoorUsePct);
                }
                set
                {
                    int num = cNumFortranProviders;
                    int pdefault = ReclaimedOutdoorUsePct;
                    citiModel = value;
                    setReclaimedOutdoorUse_(ref pdefault,ref num, ExpandProvNames35(citiModel));
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getReclaimedOutdoorUse_(ref int count, int[] values);

            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern void setReclaimedOutdoorUse_(ref int mydefault,ref int count, int[] values);
            //
            //
            internal int[] get_parmWaterSupplyToResidentialPct
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] get_parmWaterSupplyToResidentialPct = new int[num];
                    getParmWStoResPct_(ref num, get_parmWaterSupplyToResidentialPct);
                    return TrimProvNameS(get_parmWaterSupplyToResidentialPct);
                }
                set
                {
                    int num = cNumFortranProviders;
                    int pdefault = WaterSupplyToResidentialPct;
                    citiModel = value;
                    setParmWStoResPct_(ref pdefault, ref num, ExpandProvNames35(citiModel));
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getParmWStoResPct_(ref int count, int[] values);

            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern void setParmWStoResPct_(ref int mydefault,ref int count, int[] values);
            //
            internal int[] get_parmWaterSupplyToCIOpct
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] get_parmWaterSupplyToCIOpct = new int[num];
                    getParmWStoCioPct_(ref num, get_parmWaterSupplyToCIOpct);
                    return TrimProvNameS(get_parmWaterSupplyToCIOpct);
                }
                set
                {
                    int num = cNumFortranProviders;
                    int pdefault = WaterSupplyToCIOpct;
                    citiModel = value;
                    setParmWStoCioPct_(ref pdefault,ref num, ExpandProvNames35(citiModel));
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getParmWStoCioPct_(ref int count, int[] values);

            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern void setParmWStoCioPct_(ref int mydefault, ref int count, int[] values);
            //
            //
            internal int[] get_parmGroundwaterToGWTPlantPct
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] get_parmGroundwaterToGWTPlantPct = new int[num];
                    getParmGWtoGWTPPct_(ref num, get_parmGroundwaterToGWTPlantPct);
                    return TrimProvNameS(get_parmGroundwaterToGWTPlantPct);
                }
                set
                {
                    int num = cNumFortranProviders;
                    citiModel = value;
                    setParmGWtoGWTPPct_(ref num, ExpandProvNames35(citiModel));
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getParmGWtoGWTPPct_(ref int count, int[] values);

            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern void setParmGWtoGWTPPct_(ref int count, int[] values);

            // Certified 07.05.11 das
            internal bool get_parmIncludeMeteorology
            {
                set
                {
                    bool Meteo=value;
                    setParmIncludeMeteo_(ref Meteo);
                }
            }

            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern void setParmIncludeMeteo_(ref bool value);



            #endregion
        //-----------------------------------------------------------------------------------//
        #region Outputs
            internal int[] GroundwaterBankUsed  // 
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] GroundwaterBankUsed = new int[num];
                    getGWbankedUsed_(ref num, GroundwaterBankUsed);
                    return TrimProvNames(GroundwaterBankUsed);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getGWbankedUsed_(ref int count, int[] values);
            //
            internal int[] GroundwaterBankBalance  // 
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] GroundwaterBankBalance = new int[num];
                    getGWbanked_(ref num, GroundwaterBankBalance);
                    return TrimProvNames(GroundwaterBankBalance);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getGWbanked_(ref int count, int[] values);
            //


            //
            internal int[] ReclaimedWaterCreated  // 
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] ReclaimedWaterCreated = new int[num];
                    getReclaimedWaterOut_(ref num, ReclaimedWaterCreated);
                    return TrimProvNames(ReclaimedWaterCreated);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getReclaimedWaterOut_(ref int count, int[] values);
            //
            internal int[] ReclaimedWaterUsed  // 
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] ReclaimedWaterUsed = new int[num];
                    getReclaimedWaterUsed_(ref num, ReclaimedWaterUsed);
                    return TrimProvNames(ReclaimedWaterUsed);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getReclaimedWaterUsed_(ref int count, int[] values);
            //
            internal int[] ReclaimedWaterDifference  // 
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] ReclaimedWaterDifference = new int[num];
                    getReclaimedDifference_(ref num, ReclaimedWaterDifference);
                    return TrimProvNames(ReclaimedWaterDifference);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
        protected static extern int getReclaimedDifference_(ref int count, int[] values);


            //
            internal int[] ReclaimedWaterToVadose  // 
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] ReclaimedWaterToVadose = new int[num];
                    getReclaimedWaterToVadose_(ref num, ReclaimedWaterToVadose);
                    return TrimProvNames(ReclaimedWaterToVadose);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getReclaimedWaterToVadose_(ref int count, int[] values);
            //
            internal int[] ReclaimedWaterDischarged  // 
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] ReclaimedWaterDischarged = new int[num];
                    getReclaimedWaterDischarged_(ref num, ReclaimedWaterDischarged);
                    return TrimProvNames(ReclaimedWaterDischarged);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getReclaimedWaterDischarged_(ref int count, int[] values);
            //
            internal int[] ReclaimedWaterDirectInject  // 
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] ReclaimedWaterDirectInject = new int[num];
                    getReclaimedWaterDInject_(ref num, ReclaimedWaterDirectInject);
                    return TrimProvNames(ReclaimedWaterDirectInject);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getReclaimedWaterDInject_(ref int count, int[] values);
            //
            internal int[] ROreclaimedWaterCreated  // 
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] ROreclaimedWaterCreated = new int[num];
                    getROReclaimedWaterOut_(ref num, ROreclaimedWaterCreated);
                    return TrimProvNames(ROreclaimedWaterCreated);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getROReclaimedWaterOut_(ref int count, int[] values);
            //
            internal int[] ROreclaimedWaterUsed  // 
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] ROreclaimedWaterUsed = new int[num];
                    getROReclaimedWaterUsed_(ref num, ROreclaimedWaterUsed);
                    return TrimProvNames(ROreclaimedWaterUsed);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getROReclaimedWaterUsed_(ref int count, int[] values);
            //
            internal int[] ROreclaimedWaterDirectInject  // 
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] ROreclaimedWaterDirectInject = new int[num];
                    getROReclaimedWaterDI_(ref num, ROreclaimedWaterDirectInject);
                    return TrimProvNames(ROreclaimedWaterDirectInject);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getROReclaimedWaterDI_(ref int count, int[] values);
            //
            internal int[] EffluentCreated  // 
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] EffluentCreated = new int[num];
                    getEffluentCreated_(ref num, EffluentCreated);
                    return TrimProvNames(EffluentCreated);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getEffluentCreated_(ref int count, int[] values);
            //
            internal int[] EffluentToPowerPlant  // 
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] EffluentToPowerPlant = new int[num];
                    getEffluentToPowerPlant_(ref num, EffluentToPowerPlant);
                    return TrimProvNames(EffluentToPowerPlant);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getEffluentToPowerPlant_(ref int count, int[] values);
            //
            internal int[] EffluentToVadose  // 
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] EffluentToVadose = new int[num];
                    getEffluentToVadose_(ref num, EffluentToVadose);
                    return TrimProvNames(EffluentToVadose);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getEffluentToVadose_(ref int count, int[] values);
            //
            internal int[] EffluentDischarged  // 
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] EffluentDischarged = new int[num];
                    getEffluentToDischarge_(ref num, EffluentDischarged);
                    return TrimProvNames(EffluentDischarged);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getEffluentToDischarge_(ref int count, int[] values);
            //
            internal int EffluentToAgriculture
            {
                get
                {
                    int gvalue = 0;
                    return getEffluentToAg_(ref gvalue);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getEffluentToAg_(ref int value);
            //

            internal int[] VadoseToAquiferFlux  // 
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] VadoseToAquiferFlux = new int[num];
                    getfluxVadoseToAquifer_(ref num, VadoseToAquiferFlux);
                    return TrimProvNames(VadoseToAquiferFlux);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll", CallingConvention = CallingConvention.Cdecl)]
            protected static extern int getfluxVadoseToAquifer_(ref int count, int[] values);

            //
            internal int[] DemandDeficit  // 
            {
                get
                {
                    int num = cNumFortranProviders;
                    int[] DemandDeficit = new int[num];
                    getDemandDeficit_(ref num, DemandDeficit);
                    return TrimProvNames(DemandDeficit);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            protected static extern int getDemandDeficit_(ref int count, int[] values);


            #endregion
        #endregion
        //-----------------------------------------------------------------------------------//
        #region Provider Outputs
        //

        //
        #region surface water
        // CO River
        //
        // ---------------------------------------------------------------------------
        //  Annual deliveries of CO river water via CAP designations (acft a-1)
        //
        //              [CERTIFIED- 02.09.10 das] 
        internal  int[] ColoradoAnnualDeliveries  // af for the year by provider
        {
            get
            {
                int numProviders = cNumFortranProviders;
                int[] ColoradoAnnualDeliveries = new int[numProviders];
                getCOAnnualDeliveries_(ref numProviders, ColoradoAnnualDeliveries);

                return TrimProvNames(ColoradoAnnualDeliveries); // Contract 35 to 33
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void getCOAnnualDeliveries_(ref int count, int[] values);
        //
        //           [CERTIFIED- 02.15.10 das]
        internal  int[] ColoradoMaxDeliveries  // af for the year by provider
        {
            get
            {
                int numProviders = cNumFortranProviders;
                int[] ColoradoMaxDeliveries = new int[numProviders];
                getCOMaxDeliveries_(ref numProviders, ColoradoMaxDeliveries);

                return TrimProvNames(ColoradoMaxDeliveries); // Contract 35 to 33
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void getCOMaxDeliveries_(ref int count, int[] values);
        //

        // SVT Rivers
        // ---------------------------------------------------------------------------
        // 09.10.10 check these....... das
        //  Annual deliveries of SVT river water via Class A,B,and C designations (acft a-1)
        // NOT CERTIFIED
        //
        internal  int[] SVTAnnualDeliveriesSRP  // af for the year by provider
        {
            get
            {
                int num = 10;
                int[] SVAnnualDeliveries = new int[num];
                getSRPAnnualDeliveries_(ref num, SVAnnualDeliveries);
                int[] SVTAnnualDeliveriesSRP = ExpandToAllProviders(SVAnnualDeliveries); // Expand 10 to 33
                return SVTAnnualDeliveriesSRP;
                //return TrimProvNames(SVTAnnualDeliveriesSRP); // Redundant for consistency-Trim to 33
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void getSRPAnnualDeliveries_(ref int count, int[] values);
        //
        //
        //          [CERTIFIED 02.04.2010 das]
        //
        //  Maximum, class A,B,C, and NCS
        // return to this......... today
        internal  int[] SaltVerdeMaxDeliveries   // af for the year by provider
        {
            get
            {
                // 
                int numProviders = 10;
                int[] SaltVerdeMaxDeliveries10 = new int[numProviders];
                getSVMaxDeliveriesSRP_(ref numProviders, SaltVerdeMaxDeliveries10);
                int[] SaltVerdeMaxDeliveries = ExpandToAllProviders(SaltVerdeMaxDeliveries10); // Expand 10 to 33

                return SaltVerdeMaxDeliveries;
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getSVMaxDeliveriesSRP_(ref int count, int[] values);
        //

        // Class A
        //
        //              [CERTIFIED 02.04.2010 das]
        internal  int[] SaltVerdeClassADesignations   // af for the year by provider
        {
            get
            {
                // enumSRPProviders);
                int numProviders = 10;
                int[] SVClassADesignations = new int[numProviders];
                getSVClassADesignations_(ref numProviders, SVClassADesignations);

                int[] SaltVerdeClassADesignations = ExpandToAllProviders(SVClassADesignations); // Expand 10 to 33

                return SaltVerdeClassADesignations;
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getSVClassADesignations_(ref int count, int[] values);
        //

        // maximum A
        internal int[] SaltVerdeClassAmax   // af for the year by provider
        {
            get
            {
                // enumSRPProviders);
                int numProviders = 10;
                int[] SVClassAmax = new int[numProviders];
                getSVClassAmax_(ref numProviders, SVClassAmax);

                int[] SaltVerdeClassAmax = ExpandToAllProviders(SVClassAmax); // Expand 10 to 33

                return SaltVerdeClassAmax;
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getSVClassAmax_(ref int count, int[] values);
        //

        // Class B and C
        internal int[] SaltVerdeClassBCDesignations   // af for the year by provider
        {
            get
            {
                // 
                int numProviders = 10;
                int[] SVClassBC = new int[numProviders];
                getSVClassBCDesignations_(ref numProviders, SVClassBC);

                int[] SaltVerdeClassBCDesignations = ExpandToAllProviders(SVClassBC); // Expand 10 to 33

                return SaltVerdeClassBCDesignations;
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getSVClassBCDesignations_(ref int count, int[] values);
        //
        // Class B and C maximum designations
        internal int[] SaltVerdeClassBCmax   // af for the year by provider
        {
            get
            {
                // 
                int numProviders = 10;
                int[] SVClassBCmax = new int[numProviders];
                getSVClassBCmax_(ref numProviders, SVClassBCmax);

                int[] SaltVerdeClassBCmax = ExpandToAllProviders(SVClassBCmax); // Expand 10 to 33

                return SaltVerdeClassBCmax;
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getSVClassBCmax_(ref int count, int[] values);
        //

        // NCS
        //
        internal int[] SaltVerdeNCSDesignations   // af for the year by provider
        {
            get
            {
                // 
                int num = 10;
                int[] SVTNCSDesignations = new int[num];

                getSVNCS_(ref num, SVTNCSDesignations);
                int[] SaltVerdeNCSDesignations = ExpandToAllProviders(SVTNCSDesignations); // expand 10 to 33
                return SaltVerdeNCSDesignations;
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getSVNCS_(ref int count, int[] values);
        //
        internal int[] SaltVerdeNCSmax   // af for the year by provider
        {
            get
            {
                // This was brought to 35 in the FORTRAN dll
                int numProviders = 10;
                int[] SVTNCSmax = new int[numProviders];

                getSVNCSmax_(ref numProviders, SVTNCSmax);
                int[] SaltVerdeNCSmax = ExpandToAllProviders(SVTNCSmax); // expand 10 to 33
                return SaltVerdeNCSmax;
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getSVNCSmax_(ref int count, int[] values);
        //

        //   Provider surface water designations.  This includes CAP water but also surface water
        // delivered to SRP customers via the Class A designations for the 10 cities that use
        // SVT river water.
        //
        //                [CERTIFIED- 02.09.10 das] 
        internal  int[] ProviderSWdesignations  // af for the year by provider
        {
            get
            {
                int numProviders = cNumFortranProviders;
                int[] ProviderSWdesignations = new int[numProviders];
                getProviderSWDesignations_(ref numProviders, ProviderSWdesignations);

                return TrimProvNames(ProviderSWdesignations);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void getProviderSWDesignations_(ref int count, int[] values);
        //

        #endregion
        #region Water Available and that Used
        //
        //   Provider water use.  May be annual deliveries, may be demand when water demand is
        // less than supply.  This variable may need rethinking.
        //
        // NOT CERTIFIED
        internal int[] ProviderWaterUsed  // af for the year by provider
        {
            get
            {
                int numProviders = cNumFortranProviders;
                int[] ProviderWaterUsed = new int[numProviders];
                getProviderDeliveriesUsed_(ref numProviders, ProviderWaterUsed);

                return TrimProvNames(ProviderWaterUsed); // Contract 35 to 33
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void getProviderDeliveriesUsed_(ref int count, int[] values);
        //

        #endregion
        #region Groundwater

        // Provider gtoundwater designations.  They my pump up to but not exceeding this value.
        //  Originally this was addded as a check of the provider water balance
        //
        //              [CERTIFIED- 02.09.10 das] 
        internal  int[] ProviderGWdesignations  // af for the year by provider
        {
            get
            {
                int numProviders = cNumFortranProviders;
                int[] ProviderGWdesignations = new int[numProviders];
                getProviderGWDesignations_(ref numProviders, ProviderGWdesignations);

                return TrimProvNames(ProviderGWdesignations);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void getProviderGWDesignations_(ref int count, int[] values);
        //
        //
        //  The amount of groundwater pumped by each provider NOT including the SRP
        // pumping for those providers that have such designations.
        //
        // 
        //
        //           
        internal  int[] ProviderGWPumpedMunicipal  // af for the year by provider
        {
            get
            {
                int numProviders = cNumFortranProviders;
                int[] ProviderGWPumpedMunicipal = new int[numProviders];
                getProviderGWPumpageMuni_(ref numProviders, ProviderGWPumpedMunicipal);

                return TrimProvNames(ProviderGWPumpedMunicipal);

            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void getProviderGWPumpageMuni_(ref int count, int[] values);
        //
        //
        // trimmed to 33 by SaltVerdeClassBCDesignations.Length
        internal void BCtoSRP()
        {
            for (int i = 0; i < SaltVerdeClassBCDesignations.Length; ++i)
            {
                int[] ProviderGWPumpedSRP = new int[SaltVerdeClassBCDesignations.Length];
                ProviderGWPumpedSRP[i] = SaltVerdeClassBCDesignations[i];
            }
        }

        //
        // The provider specific amount of water recharged. When supply > demand.
        //
        //          [CERTIFIED- 02.10.10 das] 
        internal  int[] ProviderGWRecharge  // af for the year by provider
        {
            get
            {
                int numProviders = cNumFortranProviders;
                int[] ProviderGWRecharge = new int[numProviders];
                getProviderGWRecharge_(ref numProviders, ProviderGWRecharge);

                return TrimProvNames(ProviderGWRecharge);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void getProviderGWRecharge_(ref int count, int[] values);
        //

        // ===================================

        //
        //  As the name implies.  Calculations done in CitiWaterBudgets.f90 module.
        // Needs flushing out just, exactly, how we will define it.
        //
        internal  int[] ProviderGroundwater
        {
            get
            {
                int numProviders = cNumFortranProviders;
                int[] ProviderGroundWater = new int[numProviders];
                getAnnualProviderGroundW_(ref numProviders, ProviderGroundWater);
                return TrimProvNames(ProviderGroundWater);
            }
            set
            {
                int numProviders = cNumFortranProviders;
                int[] ProviderGroundwater = new int[numProviders];
                setProviderGroundwater_(ref numProviders, ProviderGroundwater);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void getAnnualProviderGroundW_(ref int count, int[] values);
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setProviderGroundwater_(ref int count, int[] values);
        #endregion
        //
        #endregion
        //-----------------------------------------------------------------------------------//
        #region Annual Results Variables [certified]

        #region Water Demand
        //
        //   Annual water demands for each provider.  At present, no provider water policies
        // are available, so this represents the raw empirical demand adjusted for using
        // current GPCD estimates for each provider.
        //
        //      [CERTIFIED- 02.05.10 das]
        internal  int[] WaterDemand  // af for the year by provider
        {
            get
            {
                int numProviders = cNumFortranProviders;
                int[] totalDemands = new int[numProviders];
                getTotalDemands_(ref numProviders, totalDemands);

                return TrimProvNames(totalDemands);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void getTotalDemands_(ref int count, int[] values);
        //

        //
        //    As the name implies.
        //  gv_AdjustedGPCD=nint(vPV%oDemandNoAg_acft(k)* 1/T%days *gv_galperacft*1./vPV%oPopulations(k))
        //
        // This needs resolving.  To Get ProviderGPCD, estimates would be based on demand and population.  As
        // it currently is, setting ProviderGPCD means.....? Not connected at the moment for setting...
        //
        //  [CERTIFIED- 02.05.10 das]
        internal  int[] get_ProviderGPCD
        {
            get
            {
                int numProviders = cNumFortranProviders;
                int[] ProviderGPCD = new int[numProviders];
                getProviderGPCD_(ref numProviders, ProviderGPCD);
                return TrimProvNames(ProviderGPCD);
            }
            set
                {
                  int num = cNumFortranProviders;
                  citiModel = value;
                  setProviderGPCD_(ref num,  ExpandProvNames35(citiModel));
                }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getProviderGPCD_(ref int count, int[] values);

        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern void setProviderGPCD_(ref int count, int[] values);

        #endregion

        #region Water Supply Outputs [certified]
        //
        // absent: internal  int PreMaricopaSurfaceWaterSupply
        //

         // absent: internal  int AnnualMaricopaWaterDemand
        //
        #endregion

        #region CO River variables [certified]
        //

        //
        // CO river flow.  We need to add a variable to select the data set that we want to read.
        //

        // [CERTIFIED 02.02.2010 das]
        internal  int ColoradoRiverFlow
        {
            get
            {
                int gvalue = 0;
                return getCORiverFlow_(ref gvalue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getCORiverFlow_(ref int values);
        //

        // Storage in Mead and in Powell (acft)
        //
        //       [CERTIFIED- 02.04.10 das]
        internal  int ColoradoStorage
        {
            get
            {
                int gvalue = 0;
                return getCOStorage_(ref gvalue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getCOStorage_(ref int values);
        //

        // Annual storage in Lake Powell (acft)
        //
        //      [CERTIFIED- 02.04.10 das]
        internal  int PowellStorage
        {
            get
            {
                int gvalue = 0;
                return getPowellStorage_(ref gvalue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getPowellStorage_(ref int values);
        //

        // Annual storage in Lake Mead (acft)
        //
        //  [CERTIFIED- 02.04.10 das]
        internal  int MeadStorage
        {
            get
            {
                int gvalue = 0;
                return getMeadStorage_(ref gvalue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getMeadStorage_(ref int values);
        //

        // CAP delivery (county scale) Deliveries into Maricopa County.
        //
        //   [CERTIFIED- 02.04.10 das]
        internal  int ColoradoDelivery
        {
            get
            {
                int gvalue = 0;
                return getCODelivery_(ref gvalue);

            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getCODelivery_(ref int values);
        //
        //
        // absent:  internal  int CAP2MandIexcess

        //    [CERTIFIED- 02.15.10 das]
        internal  int CAP2MandI
        {
            get
            {
                int gvalue = 0;
                return getCAPMandI_(ref gvalue);

            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getCAPMandI_(ref int values);
        //
        internal  int AZshareCO
        {
            get
            {
                int gvalue = 0;
                return getAZshareCO_(ref gvalue);

            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getAZshareCO_(ref int values);
        //
        // absent: 
        //

        #endregion

        #region Salt-Verde-Tonto River Variables [certified]
        //

        // SRP release from the reservoir system.  This is a function of storage and flow. Annual
        // estimates are, just that, estimates.  Monthly will be available soon, using actual release
        // estimates.
        //
        //   [CERTIFIED- 02.04.10 das]
        internal  int SaltVerdeDelivery
        {
            get
            {
                int gvalue = 0;
                return getSVDelivery_(ref gvalue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getSVDelivery_(ref int values);
        //

        //   Salt-Verde-Tonto River flow.  I added a variable to select the data set that we
        // want to use.  At present it defaults to the Bureau of Reclamation data.
        //
        //              [CERTIFIED 02.02.2010 das]
        internal  int SaltVerdeRiverFlow
        {
            get
            {
                int gvalue = 0;
                return getSVRiverFlow_(ref gvalue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getSVRiverFlow_(ref int values);
        //

        // Storage on the Salt, Verde, and Tonto Rivers. We model only one reservoir (acft)
        //
        //    [CERTIFIED 02.04.2010 das]
        internal  int SaltVerdeStorage
        {
            get
            {
                int gvalue = 0;
                return getSVStorage_(ref gvalue);
            }
        }
        [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
        protected static extern int getSVStorage_(ref int values);
        //
        //
        // --------------------------------------------------------------

        //
        // absent:  internal  int SaltVerdeAnnualGroundwaterDemand 
        //

        //
        // absent: internal  int SaltVerdeRechargeSRVReach 
        //

        #endregion

        #endregion
        //-----------------------------------------------------------------------------------//
        //
        #region Utilities
        // ------------------------------------------------------------------------------

        internal delegate void RunCallbackHandler(int year);
        internal enum ShortageSharingPolicyType { ProportionalSharing = 1, AgLosesWaterFirst };

        internal virtual int RangeCheckShortageSharingPolicy(ShortageSharingPolicyType value)
        {
            return RangeCheck((int)ShortageSharingPolicyType.ProportionalSharing, (int)value,
                (int)ShortageSharingPolicyType.AgLosesWaterFirst,
                "ShortageSharingPolicy");
        }
        //
        const int AdjNumProviders = 33;

        protected int[] TrimProvNames(int[] OrigArray)
        {
            int[] AdjArray = new int[AdjNumProviders];
            int iAdjArray = 0;
            for (int iOrigArray = 0; iOrigArray < OrigArray.Length; ++iOrigArray)
            {
                if (0 < ProvNameIndicies[iOrigArray]) AdjArray[iAdjArray++] = OrigArray[iOrigArray];
            }
            if (AdjNumProviders != iAdjArray) throw new Exception("Provider Array Lengths Not Copacetic-537 in UG.cs");
            return AdjArray;
        }
        protected static int[] TrimProvNameS(int[] OrigArray)
        {
            int[] AdjArray = new int[AdjNumProviders];
            int iAdjArray = 0;
            for (int iOrigArray = 0; iOrigArray < OrigArray.Length; ++iOrigArray)
            {
                if (0 < ProvNameIndicies[iOrigArray]) AdjArray[iAdjArray++] = OrigArray[iOrigArray];
            }
            if (AdjNumProviders != iAdjArray) throw new Exception("Provider Array Lengths Not Copacetic-537 in UG.cs");
            return AdjArray;
        }

        internal static int[] ProvNameIndicies = new int[]
        {
             1,  // Adaman Mutual
             1,  // Arizona Water Company - White Tanks
             1,  // Arizona-American - Paradise Valley
             1,  // Arizona-American - Sun City
             1,  // Arizona-American - Sun City West
             1,  // Avondale
             1,  // Berneil
             1,  // Buckeye
             1,  // Carefree
             1,  // Cave Creek
             1,  // Chandler
             1,  // Chaparral City
             1,  // City of Surprise
             1,  // Clearwater Utilities
             1,  // Desert Hills
             1,  // El Mirage
             1,  // Gilbert
             1,  // Glendale
             1,  // Goodyear
             1,  // Litchfield Park
             1,  // Mesa
            -1,  // no provider
            -1,  // other provider
             1,  // Peoria
             1,  // Phoenix
             1,  // Queen Creek
             1,  // Rigby
             1,  // Rio Verde
             1,  // Rose Valley
             1,  // Scottsdale
             1,  // Sunrise
             1,  // Tempe
             1,  // Tolleson
             1,  // Valley Utilities
             1  // West End
         };
        internal static int[] ProvNameIndicies35 = new int[]
        {
             1,  // Adaman Mutual
             1,  // Arizona Water Company - White Tanks
             1,  // Arizona-American - Paradise Valley
             1,  // Arizona-American - Sun City
             1,  // Arizona-American - Sun City West
             1,  // Avondale
             1,  // Berneil
             1,  // Buckeye
             1,  // Carefree
             1,  // Cave Creek
             1,  // Chandler
             1,  // Chaparral City
             1,  // City of Surprise
             1,  // Clearwater Utilities
             1,  // Desert Hills
             1,  // El Mirage
             1,  // Gilbert
             1,  // Glendale
             1,  // Goodyear
             1,  // Litchfield Park
             1,  // Mesa
            -1,  // no provider
            -1,  // other provider 
             1,  // Peoria
             1,  // Phoenix
             1,  // Queen Creek
             1,  // Rigby
             1,  // Rio Verde
             1,  // Rose Valley
             1,  // Scottsdale
             1,  // Sunrise
             1,  // Tempe
             1,  // Tolleson
             1,  // Valley Utilities
             1,  // West End
            -1   // off-set
         };
         protected double[] TrimProvNamesD35(double[] OrigArray)
        {
            double[] AdjArray = new double[AdjNumProviders];
            int iAdjArray = 0;
            for (int iOrigArray = 0; iOrigArray < OrigArray.Length; ++iOrigArray)
            {
                if (0 < ProvNameIndicies[iOrigArray]) AdjArray[iAdjArray++] = OrigArray[iOrigArray];
            }
            if (AdjNumProviders != iAdjArray) throw new Exception("Provider Array Lengths Not Copacetic-537 in UG.cs");
            return AdjArray;
        }

        internal static double[] TrimProvNamesD38(double[] OrigArray)
        {
            double[] AdjArray = new double[33];
            int iAdjArray = 0;
            for (int iOrigArray = 0; iOrigArray < OrigArray.Length; ++iOrigArray)
            {
                if (0 < ProvNameIndiciesAll[iOrigArray]) AdjArray[iAdjArray++] = OrigArray[iOrigArray];
            }
            if (AdjNumProviders != iAdjArray) throw new Exception("Provider Array Lengths Not Copacetic-537 in UG.cs");
            return AdjArray;
        }
        internal static int[] ExpandProvNames35(int[] OrigArray)
        {
            int[] AdjArray = new int[36];
            int iArray = 33;
            int icount = 0;
            for (int iAdjArray = 0; iAdjArray < AdjArray.Length; ++iAdjArray)
            {
                if (0 < ProvNameIndicies35[iAdjArray])
                {

                    AdjArray[iAdjArray] = OrigArray[icount];
                    ++icount;
                }
            }
            if (icount != iArray) throw new Exception("Provider Array Lengths Not Copacetic-1530 in UG.cs");
            return AdjArray;
        }

        internal static double[] ExpandProvNames(double[] OrigArray)
        {
            double[] AdjArray = new double[38];
            int iArray = 33;
            int icount = 0;
            for (int iAdjArray = 0; iAdjArray < AdjArray.Length; ++iAdjArray)
            {
                if (0 < ProvNameIndiciesAll[iAdjArray])
                {
                    AdjArray[iAdjArray] = OrigArray[icount];
                    ++icount;
                }
            }
            if (icount != iArray) throw new Exception("Provider Array Lengths Not Copacetic-1530 in UG.cs");
            return AdjArray;
        }
        internal static int[] ExpandProvNamesInt(int[] OrigArray)
        {
            int[] AdjArray = new int[38];
            int iArray = 33;
            int icount = 0;
            for (int iAdjArray = 0; iAdjArray < AdjArray.Length; ++iAdjArray)
            {
                if (0 < ProvNameIndiciesAll[iAdjArray])
                {
                    AdjArray[iAdjArray] = OrigArray[icount];
                    ++icount;
                }
            }
            if (icount != iArray) throw new Exception("Provider Array Lengths Not Copacetic-1546 in UG.cs");
            return AdjArray;
        }

        internal static int[] ProvNameIndiciesAll = new int[]
        {
             1,  // Adaman Mutual
             1,  // Arizona Water Company - White Tanks
             1,  // Arizona-American - Paradise Valley
             1,  // Arizona-American - Sun City
             1,  // Arizona-American - Sun City West
             1,  // Avondale
             1,  // Berneil
             1,  // Buckeye
             1,  // Carefree
             1,  // Cave Creek
             1,  // Chandler
             1,  // Chaparral City
             1,  // City of Surprise
             1,  // Clearwater Utilities
             1,  // Desert Hills
             1,  // El Mirage
             1,  // Gilbert
             1,  // Glendale
             1,  // Goodyear
             1,  // Litchfield Park
             1,  // Mesa
            -1,  // no provider
            -1,  // other provider
             1,  // Peoria
             1,  // Phoenix
             1,  // Queen Creek
             1,  // Rigby
             1,  // Rio Verde
             1,  // Rose Valley
             1,  // Scottsdale
             1,  // Sunrise
             1,  // Tempe
             1,  // Tolleson
             1,  // Valley Utilities
             1,  // West End
            -1,  //  "Gila River Reservation",
            -1,  // "Salt River Reservation",
            -1   // "Salt River Project"
         };
        //internal enum providers
        //{
        //     Adaman_Mutual,
        //     White_Tanks,
        //     Paradise_Valley,
        //     Sun_City,
        //     Sun_City_West,
        //     Avondale,
        //     Berneil,
        //     Buckeye,
        //     Carefree,
        //     Cave_Creek,
        //     Chandler,
        //     Chaparral_City,
        //     Surprise,
        //     Clearwater_Utilities,
        //     Desert_Hills,
        //     El_Mirage,
        //     Gilbert,
        //     Glendale,
        //     Goodyear,
        //     Litchfield_Park,
        //     Mesa,
        //     NoProvider,
        //     OtherProvider,
        //     Peoria,
        //     Phoenix,
        //     Queen_Creek,
        //     Rigby,
        //     Rio_Verde,
        //     Rose_Valley,
        //     Scottsdale,
        //     Sunrise,
        //     Tempe,
        //     Tolleson,
        //     Valley_Utilities,
        //     West_End

        //};
        #endregion
        //
        #region Shared Utility Routines

        internal virtual int[] ExpandToAllProviders(int[] srpValuesOnly)
        {
            if (null == srpValuesOnly) return null;
            int[] fullList = new int[SRPProviderNameIndices.Length];
            for (int iProv = 1; iProv < SRPProviderNameIndices.Length; ++iProv)
            {
                fullList[iProv] = 0 > SRPProviderNameIndices[iProv] ? 0 :
                    srpValuesOnly[SRPProviderNameIndices[iProv]];
            }
            return fullList;
        }
        protected static int[] SRPProviderNameIndices = new int[]
        #region Offsets into SRP-only providers list
        {
            -1,  // ad   0 Adaman Mutual
            -1,  // wt   4 Arizona Water Company - White Tanks
            -1,  // pv   1 Arizona-American - Paradise Valley
            -1,  // su   2 Arizona-American - Sun City
            -1,  // sw   3 Arizona-American - Sun City West
             0,  // av   5 Avondale
            -1,  // be   6 Berneil
            -1,  // bu   7 Buckeye
            -1,  // cf   8 Carefree
            -1,  // cc   9 Cave Creek
             1,  // ch  10 Chandler
            -1,  // cp  11 Chaparral City
            -1,  // sp  12 City of Surprise
            -1,  // cu  13 Clearwater Utilities
            -1,  // dh  14 Desert Hills
            -1,  // em  15 El Mirage
             2,  // gi  16 Gilbert
             3,  // gl  17 Glendale
            -1,  // go  18 Goodyear
            -1,  // lp  19 Litchfield Park
             4,  // me  20 Mesa
             5,  // pe  21 Peoria
             6,  // ph  22 Phoenix
            -1,  // qk  23 Queen Creek
            -1,  // rg  24 Rigby
            -1,  // rv  25 Rio Verde
            -1,  // ry  26 Rose Valley
             7,  // sc  27 Scottsdale
            -1,  // sr  28 Sunrise
             8,  // te  29 Tempe
             9,  // to  30 Tolleson
            -1,  // vu  31 Valley Utilities
            -1   // we  32 West End
        };
        #endregion

        protected string[] enumProviders =
        #region Provider names in the order that WaterSim maintains them for use in enumerated arrays
        {
            "Adaman Mutual",
            "Arizona Water Company - White Tanks",
            "Arizona-American - Paradise Valley",
            "Arizona-American - Sun City",
            "Arizona-American - Sun City West",
            "Avondale_",
            "Berneil_",
            "Buckeye_",
            "Carefree_",
            "Cave Creek_",
            "Chandler_",
            "Chaparral City_",
            "City of Surprise",
            "Clearwater Utilities",
            "Desert Hills_",
            "El Mirage_",
            "Gilbert_",
            "Glendale_",
            "Goodyear_",
            "Litchfield Park_",
            "Mesa_",
            "no Provider",
            "other Mini Providers",
            "Peoria_",
            "Phoenix_",
            "Queen Creek_",
            "Rigby_",
            "Rio Verde_",
            "Rose Valley_",
            "Scottsdale_",
            "Sunrise_",
            "Tempe_",
            "Tolleson_",
            "Valley Utilities",
            "West End"
        };
        #endregion

        internal string[] ProviderNames
        {
            get
            {
                string[] providerNames = new string[]
                #region Provider names in the order that WaterSim maintains them
                {
                    "Adaman Mutual",
                    "Arizona Water Company - White Tanks",
                    "Arizona-American - Paradise Valley",
                    "Arizona-American - Sun City",
                    "Arizona-American - Sun City West",
                    "Avondale",
                    "Berneil",
                    "Buckeye",
                    "Carefree",
                    "Cave Creek",
                    "Chandler",
                    "Chaparral City",
                    "City of Surprise",
                    "Clearwater Utilities",
                    "Desert Hills",
                    "El Mirage",
                    "Gilbert",
                    "Glendale",
                    "Goodyear",
                    "Litchfield Park",
                    "Mesa",
                    "no provider",
                    "other providers",
                    "Peoria",
                    "Phoenix",
                    "Queen Creek",
                    "Rigby",
                    "Rio Verde",
                    "Rose Valley",
                    "Scottsdale",
                    "Sunrise",
                    "Tempe",
                    "Tolleson",
                    "Valley Utilities",
                    "West End"
                };
                #endregion
                return providerNames;
            }
        }

        protected string[] enumSRPProviders =
        #region SRP Provider names in the order that WaterSim maintains them for use in enumerated arrays
        {
            "Avondale_SRP",
            "Chandler_SRP",
            "Gilbert_SRP",
            "Glendale_SRP",
            "Mesa_SRP",
            "Peoria_SRP",
            "Phoenix_SRP",
            "Scottsdale_SRP",
            "Tempe_SRP",
            "Tolleson_SRP"
        };
        #endregion

        internal string[] SRPProviderNames
        {
            get
            {
                string[] srpProviderNames = new string[]
                #region Provider names in the order that WaterSim maintains them
                {
                    "Avondale",
                    "Chandler",
                    "Gilbert",
                    "Glendale",
                    "Mesa",
                    "Peoria",
                    "Phoenix",
                    "Scottsdale",
                    "Tempe",
                    "Tolleson",
                };
                #endregion
                return srpProviderNames;
            }
        }
        internal int ToInt(double value)
        {
            return Convert.ToInt32(Math.Round(value));
        }


      #region RangeChecks
        #region Simulation Duration
            internal virtual int MinStartYear
            { get { return 2000; } }

            internal virtual int MaxStartYear
            { get { return 2006; } }

            internal virtual int RangeCheckStartYear(int value)
            {
                return RangeCheck(MinStartYear, value,
                    MaxStartYear,
                    "Simulation start year out-of-bounds");
            }

            //internal virtual int SimulationStart
            //{ get { return 2006; } }

            internal virtual int SimulationDuration
            { get { return 75; } }

            //internal virtual int SimulationEnd
            //{ get { return SimulationStart + SimulationDuration; } }

            internal virtual int MinEndYear
            { get { return 2007; } }

            internal virtual int MaxEndYear
            { get { return 2086; } }

            internal virtual int RangeCheckEndYear(int value)
            {
                return RangeCheck(MinEndYear, value,
                    MaxEndYear,
                    "Simulation end year out-of-bounds");
            }

        #endregion
     
        #region Colorado River Variables

        // Runoff data to use
        // -----------------------------------------------------
        internal virtual int MinColoradoHistoricalData
        { get { return 1; } } // Paleo Data

        internal virtual int MaxColoradoHistoricalData
        { get { return 3; } } // Other data (Bureau of Rec)

        internal virtual int RangeCheckColoradoData(int value)
        {
            return RangeCheck(MinColoradoHistoricalData, value,
                MaxColoradoHistoricalData,
                "ColoradoHistoricalDataUse");
        }
        //----------------------


        // Index year of historical runoff to use
        //---------------------------------------------------------
        internal virtual int MinColoradoHistoricalExtractionStartYear
        { get { return 762; } } //1906 paleo is 762;

         internal virtual int MaxColoradoHistoricalExtractionStartYear
        { get { return 1982; } }

        internal virtual int RangeCheckColoradoHistoricalExtractionStartYear(int value)
        {
            return RangeCheck(MinColoradoHistoricalExtractionStartYear, value,
                MaxColoradoHistoricalExtractionStartYear,
                "ColoradoHistoricalExtractionStartYear");
        }
         //----------------------


        // Drought year on the CO to start
        // --------------------------------------------------
        internal virtual int MinColoradoUserAdjustmentStartYear
        { get { return startyear; } }

        internal virtual int MaxColoradoUserAdjustmentStartYear
        { get { return minStopYearDrought; } }

        internal virtual int RangeCheckColoradoUserAdjustmentStartYear(int value)
        {
            return RangeCheck(MinColoradoUserAdjustmentStartYear, value,
                MaxColoradoUserAdjustmentStartYear,
                "ColoradoUserAdjustmentStartYear");
        }

         //----------------------


        // Drought year on the CO River to end
        // --------------------------------------------------------
        internal virtual int MinColoradoUserAdjustmentStopYear
        { get { return startyear; } }

        internal virtual int MaxColoradoUserAdjustmentStopYear
        { get { return minStopYearDrought; } }

        internal virtual int RangeCheckColoradoUserAdjustmentStopYear(int value)
        {
            return RangeCheck(MinColoradoUserAdjustmentStopYear, value,
                MaxColoradoUserAdjustmentStopYear,
                "ColoradoUserAdjustmentStopYear");
        }
        // --------------------


        // Severity of drought (or surplus)on the CO River
        // ----------------------------------------------------------
        internal virtual int MinColoradoUserAdjustmentPercent
        { get { return 0; } }

        internal virtual int MaxColoradoUserAdjustmentPercent
        { get { return 300; } }

        internal virtual int RangeCheckColoradoUserAdjustmentPercent(int value)
        {
            return RangeCheck(MinColoradoUserAdjustmentPercent, value,
                MaxColoradoUserAdjustmentPercent,
                "ColoradoUserAdjustmentPercent");
        }
        // -------------------

        // Climate impacts on the CO River (delta runoff)
        // internal override int ColoradoClimateAdjustmentPercent
        // ----------------------------------------------------
        internal virtual int MinColoradoClimateAdjustmentPercent
        { get { return 0; } }
        internal virtual int MaxColoradoClimateAdjustmentPercent
        { get { return 300; } }
        internal virtual int RangeCheckColoradoClimateAdjustmentPercent(int value)
        {
            return RangeCheck(MinColoradoClimateAdjustmentPercent, value,
                MaxColoradoClimateAdjustmentPercent,
                "ColoradoClimateAdjustmentPercent");
        }
        // -------------------


        // NOT USED
        // hold-over from the original WaterSim
        // --------------------------------------------------
        internal virtual int MinColoradoActualEstimateOrRandom
        { get { return 1; } }
        internal virtual int MaxColoradoActualEstimateOrRandom
        { get { return 3; } }
        internal virtual int RangeCheckColoradoActualEstimateOrRandom(int value)
        {
            return RangeCheck(MinColoradoActualEstimateOrRandom, value,
                MaxColoradoActualEstimateOrRandom,
                "ColoradoActualEstimateOrRandom");
        }
        // --------------------

        #endregion

        #region Salt/Verde River Variables
        // Runoff data to use
        // -----------------------------------------------------
        internal virtual int MinSVTHistoricalData
        { get { return 1; } } // Paleo Data

        internal virtual int MaxSVTHistoricalData
        { get { return 3; } } // Other data
        internal virtual int RangeCheckSVTData(int value)
        {
            return RangeCheck(MinSVTHistoricalData, value,
                MaxSVTHistoricalData,
                "Salt-Verde-TontoHistoricalDataUse");
        }
        //----------------------


        // Index year of historical runoff to use
        //------------------------------------------------------------
        // internal override int SaltVerdeHistoricalExtractionStartYear
        internal virtual int MinSaltVerdeHistoricalExtractionStartYear
        { get { return 1330; } } //1946 - 1330 AD is paleo;
        internal virtual int MaxSaltVerdeHistoricalExtractionStartYear
        { get { return 1982; } }

        internal virtual int RangeCheckSaltVerdeHistoricalExtractionStartYear(int value)
        {
            return RangeCheck(MinSaltVerdeHistoricalExtractionStartYear, value,
                MaxSaltVerdeHistoricalExtractionStartYear,
                "SaltVerdeHistoricalExtractionStartYear");
        }
        //----------------------

        // Drought year on the Salt-Verde to start
        // ----------------------------------------------------
        // internal override int SaltVerdeUserAdjustmentStartYear
        internal virtual int MinSaltVerdeUserAdjustmentStartYear
        { get { return startyear; } }
        internal virtual int MaxSaltVerdeUserAdjustmentStartYear
        { get { return minStopYearDrought; } }
        internal virtual int RangeCheckSaltVerdeUserAdjustmentStartYear(int value)
        {
            return RangeCheck(MinSaltVerdeUserAdjustmentStartYear, value,
                MaxSaltVerdeUserAdjustmentStartYear,
                "SaltVerdeUserAdjustmentStartYear");
        }
        //----------------------

        // Drought year on the Salt-Verde to stop
        // ----------------------------------------------------
        // internal override int SaltVerdeUserAdjustmentStopYear
        internal virtual int MinSaltVerdeUserAdjustmentStopYear
        { get { return startyear; } }

        internal virtual int MaxSaltVerdeUserAdjustmentStopYear
        { get { return minStopYearDrought; } }

        internal virtual int RangeCheckSaltVerdeUserAdjustmentStopYear(int value)
        {
            return RangeCheck(MinSaltVerdeUserAdjustmentStopYear, value,
                MaxSaltVerdeUserAdjustmentStopYear,
                "SaltVerdeUserAdjustmentStopYear");
        }
        //----------------------

        // Drought effect on runoff for the Salt-Verde 
        // ----------------------------------------------------
        // internal override int SaltVerdeUserAdjustmentPercent
        internal virtual int MinSaltVerdeUserAdjustmentPercent
        { get { return 0; } }

        internal virtual int MaxSaltVerdeUserAdjustmentPercent
        { get { return 400; } }

        internal virtual int RangeCheckSaltVerdeUserAdjustmentPercent(int value)
        {
            return RangeCheck(MinSaltVerdeUserAdjustmentPercent, value,
                MaxSaltVerdeUserAdjustmentPercent,
                "SaltVerdeUserAdjustmentPercent");
        }
        //----------------------

        // Climate effects on runoff for the Salt-Verde 
        // ---------------------------------------------------------
        //   internal override int SaltVerdeClimateAdjustmentPercent
        internal virtual int MinSaltVerdeClimateAdjustmentPercent
        { get { return 0; } }

        internal virtual int MaxSaltVerdeClimateAdjustmentPercent
        { get { return 400; } }

        internal virtual int RangeCheckSaltVerdeClimateAdjustmentPercent(int value)
        {
            return RangeCheck(MinSaltVerdeClimateAdjustmentPercent, value,
                MaxSaltVerdeClimateAdjustmentPercent,
                "SaltVerdeClimateAdjustmentPercent");
        }
        //----------------------


        // 
        // NOT USED
        // --------------------------------------------------------------------------------
        internal enum SRPReleaseEstimateMethodType { Historical = 1, TJLApproach, DASApproach };

        internal virtual int RangeCheckSRPReleaseEstimateMethod(SRPReleaseEstimateMethodType value)
        {
            return RangeCheck((int)SRPReleaseEstimateMethodType.Historical, (int)value,
                (int)SRPReleaseEstimateMethodType.DASApproach,
                "SRPReleaseEstimateMethod");
        }

        //----------------------


        // This variable now controls use of the Salt-Verde-Tonto Paleo climate data - as of 26 June 2009
        // das (i.e. can no longer use a value of two if a longer stream record is desired)
        // --------------------------------------------------------------------------------
        internal virtual int MinSaltVerdeActualEstimateOrRandom
        { get { return 1; } }

        internal virtual int MaxSaltVerdeActualEstimateOrRandom
        { get { return 3; } }

        internal virtual int RangeCheckSaltVerdeActualEstimateOrRandom(int value)
        {
            return RangeCheck(MinSaltVerdeActualEstimateOrRandom, value,
                MaxSaltVerdeActualEstimateOrRandom,
                "SaltVerdeActualEstimateOrRandom");
        }
        // --------------------------

        #endregion

        #region Population Variables

        // ----------------------------------------------------------
        internal virtual int MinPopulationGrowthRateAdjustmentPercent
        { get { return 0; } }

        internal virtual int MaxPopulationGrowthRateAdjustmentPercent
        { get { return 700; } }

        internal virtual int RangeCheckPopulationGrowthRateAdjustmentPercent(int value)
        {
            return RangeCheck(MinPopulationGrowthRateAdjustmentPercent, value,
                MaxPopulationGrowthRateAdjustmentPercent,
                "PopulationGrowthRateAdjustmentPercent");
        }

        // --------------------------

        //  NOT USED
        // ---------------------------------------------------------
        internal virtual int MinPopulationIn2030
        { get { return 3000000; } }

        internal virtual int MaxPopulationIn2030
        { get { return 30000000; } }

        internal virtual int RangeCheckPopulationIn2030(int value)
        {
            return RangeCheck(MinPopulationIn2030, value,
                MaxPopulationIn2030,
                "PopulationIn2030");
        }
        // --------------------------
        internal virtual int RangeCheckWaterPolicy(WaterPolicyType value)
        {
            return RangeCheck((int)WaterPolicyType.NoOverdraft, (int)value,
                (int)WaterPolicyType.FixedResidentialGPCD,
                "WaterPolicy");
        }

 

        #endregion

        #region GenericProvider
        // Provider level
        // Reduction in GPCD (%) by thte year 2100; used in the SES algorithm
        // ------------------------------------------------------------------
        internal virtual int MinGPCDpct
        { get { return 0; } }

        internal virtual int MaxGPCDpct
        { get { return 75; } }

        internal virtual int RangeCheckGPCDpct(int value)
        {
            return RangeCheck(MinGPCDpct, value, MaxGPCDpct, "GPCD pct reduction");
        }
        // -------------------------
        // Old code- County scale
        // ------------------------------------------------------------------
        internal virtual int MinWaterPolicyStartYear
        { get { return SetSimulationStart; } }

        internal virtual int MaxWaterPolicyStartYear
        { get { return SetSimulationEnd; } }

        internal virtual int RangeCheckWaterPolicyStartYear(int value)
        {
            return RangeCheck(MinWaterPolicyStartYear, value,
                MaxWaterPolicyStartYear,
                "WaterPolicyStartYear");
        }

        // Old code- County scale
        // internal override int ResidentialGPCD
        // ------------------------------------------------------
        internal virtual int MinResidentialGPCD
        { get { return 0; } }

        internal virtual int MaxResidentialGPCD
        { get { return 1200; } }

        internal virtual int RangeCheckResidentialGPCD(int value)
        {
            return RangeCheck(MinResidentialGPCD, value,
                MaxResidentialGPCD,
                "ResidentialGPCD");
        }
        // Old code- County scale
        //  internal override int ResidDensityPath
        // ------------------------------------------------------------------
        internal virtual int MinResidDensityPath
        { get { return 1; } }

        internal virtual int MaxResidDensityPath
        { get { return 3; } }

        internal virtual int RangeCheckResidDensityPath(int value)
        {
            return RangeCheck(MinResidDensityPath, value,
                MaxResidDensityPath,
                "ResidDensityPath");
        }
        internal virtual int MinHHoldPcts
        { get { return 0; } }

        internal virtual int MaxHHoldPcts
        { get { return 100; } }

        internal virtual int RangeCheckHHoldPct(int value)
        {
            return RangeCheck(MinHHoldPcts, value,
                MaxHHoldPcts,
                "Household percents");
        }
        // -------------------------


        // Water demand options
        // internal override int ProviderDemandOption
        // ------------------------------------------------------------------
        internal virtual int MinDemandOption
        { get { return 1; } }

        internal virtual int MaxDemandOption
        { get { return 4; } }

        internal virtual int RangeCheckDemandOption(int value)
        {
            return RangeCheck(MinDemandOption, value,
                MaxDemandOption,
                "Demand Option");
        }
        // Old code- County scale
        //  internal override int AgricultureRetirementYear
        // ------------------------------------------------------------------
        internal virtual int MinAgricultureRetirementYear
        { get { return 2020; } }

        internal virtual int MaxAgricultureRetirementYear
        { get { return 2099; } }

        internal virtual int RangeCheckAgricultureRetirementYear(int value)
        {
            return RangeCheck(MinAgricultureRetirementYear, value,
                MaxAgricultureRetirementYear,
                "AgricultureRetirementYear");
        }
        // Change water demand as a percent of read input
        // internal int DemandPercent
        // ------------------------------------------------------------------
        internal virtual int MinDemandPercent
        { get { return 1; } }

        internal virtual int MaxDemandPercent
        { get { return 300; } }

        internal virtual int RangeCheckDemandPercent(int value)
        {
            return RangeCheck(MinDemandPercent, value,
                MaxDemandPercent,
                "Demand Percent");
        }
        #endregion

        #region Generic
        internal int RangeCheck(int min, int value, int max, string tag)
        {
            if (min > value || max < value)
            {
                throw new ArgumentException(tag + " value is out of range");
            }
            return value;
        }
        int cNumFortran = 35;
        internal int[] RangeChecks(int min, int[] values, int max, string tag)
        {
            for (int num = 0; num < cNumFortran; ++num)
            {
                if (min > values[num] || max < values[num])
                {
                    throw new ArgumentException(tag + " Value is out of range");
                }
            }
            return values;
        }

        internal double RangeCheck(double min, double value, double max, string tag)
        {
            if (min > value || max < value)
            {
                throw new ArgumentException(tag + " value is out of range");
            }
            return value;
        }
        #endregion

      #endregion

        internal void NotImplementedException()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        internal int[] PopulationGrowthRateAdjust = new int[]
        {
            10,
            11,
            12,
            13,
            14,
            15,
            16,
            17,
            18,
            19,
            20,
            21,
            22,
            23,
            24,
            25,
            26,
            27,
            28,
            29,
            30,
            31,
            32,
            33,
            34,
            35,
            36,
            37,
            38,
            39,
            40,
            41,
            42,
            43,
            44
        };
      
        internal struct Passing
        {

            internal float TemperatureMultiplier
            {
                get
                {
                    int gvalue = 0;
                    float ClimateDemand = 0;
                    getDemandMultiplier_(ref gvalue);
                    return ClimateDemand = Convert.ToSingle(gvalue * 0.01);
                }
            }
            [DllImport(@"WaterSimDCDC_model.dll",CallingConvention=CallingConvention.Cdecl )]
            static extern int getDemandMultiplier_(ref int value);

        }
        //
        #endregion

    }
    }
   
